@@ -411,11 +411,17 @@ syck_emit( SyckEmitter *e, st_data_t n )
         }
         else
         {
-            if (anchor_name == NULL) { anchor_name = "1"; }
+            char *an;
+            if (anchor_name == NULL) {
+                an = S_ALLOC_N( char, x+2 );
+                sprintf(an, "*%d", x);
+            }
+            else {
+                an = S_ALLOC_N( char, strlen( anchor_name ) + 2 );
+                sprintf( an, "*%s", anchor_name );
+            }
 
-            char *an = S_ALLOC_N( char, strlen( anchor_name ) + 2 );
-            sprintf( an, "*%s", anchor_name );
-            syck_emitter_write( e, an, strlen( anchor_name ) + 1 );
+            syck_emitter_write( e, an, strlen( an ) );
             free( an );
 
             goto end_emit;
